% Nan Xiao

% 6/29/05

% assemble M K L matrices in the 'Hines' method

function [Ca_M,Ca_A] = neur_assembleCaLHS(nc,md,p_const,p_conds) 

M = spalloc(md.count.L,md.count.L,md.count.L*3-4);
K = spalloc(md.count.L,md.count.L,md.count.L*3-4);

root = md.root;
rootdex = md.rootdex;
notroot = md.notroot;
notrootdex = md.notrootdex;

Mcen = cell(md.count.N,1);
Moff = cell(md.count.N,1);

Kcen = cell(md.count.N,1);
Koff = cell(md.count.N,1);

for jj = 1:md.count.N % number of branches
    
    h = md.grid.hstep(jj);
     
    Mcen{jj} = 2/3*h*ones(1,md.interp.size(jj));
    Mcen{jj}(1) = h/3;
    Mcen{jj}(end) = h/3;
     
    Kcen{jj} = 2/h*ones(1,md.interp.size(jj));
    Kcen{jj}(1) = 1/h;
    Kcen{jj}(end) = 1/h;
    
    Moff{jj} = h/6*ones(1,md.interp.size(jj)-1);
    Koff{jj} = -1/h*ones(1,md.interp.size(jj)-1);
        
end

% first root is designated as 'master root'

jj = rootdex(end);

segh = md.segh(jj);
segt = md.segt(jj);

st = md.count.L*(segh-1)+segh;
fin = md.count.L*(segt-1)+segt;
step = md.count.L+1;

M(st:step:fin) = Mcen{jj};
M(st+1:step:fin-md.count.L) = Moff{jj};
M(st+md.count.L:step:fin-1) = Moff{jj};

K(st:step:fin) = Kcen{jj};
K(st+1:step:fin-md.count.L) = Koff{jj};
K(st+md.count.L:step:fin-1) = Koff{jj};

% rest of the roots

for ii = 1:length(rootdex)-1

    jj = rootdex(ii);
    
    segh = md.segh(jj);
    segt = md.segt(jj);
    
    st = md.count.L*(segh-1)+segh;
    fin = md.count.L*(segt-1)+segt;
    step = md.count.L+1;
    
    M(st:step:fin) = Mcen{jj}(1:end-1);
    M(st+1:step:fin-md.count.L) = Moff{jj}(1:end-1);
    M(st+md.count.L:step:fin-1) = Moff{jj}(1:end-1);
    
    M(end,end) = M(end,end) + Mcen{jj}(end);
    M(end,segt) = Moff{jj}(end);
    M(segt,end) = Moff{jj}(end);
    
    K(st:step:fin) = Kcen{jj}(1:end-1);
    K(st+1:step:fin-md.count.L) = Koff{jj}(1:end-1);
    K(st+md.count.L:step:fin-1) = Koff{jj}(1:end-1);
    
    K(end,end) = K(end,end) + Kcen{jj}(end);
    K(end,segt) = Koff{jj}(end);
    K(segt,end) = Koff{jj}(end);
       
end

% rest of the branches

for ii = 1:length(notrootdex)
   
    jj = notrootdex(ii);
    
    segh = md.segh(jj);
    segt = md.segt(jj);
    
    st = md.count.L*(segh-1)+segh;
    fin = md.count.L*(segt-1)+segt;
    step = md.count.L+1;
    
    M(st:step:fin) = Mcen{jj}(1:end-1);
    M(st+1:step:fin-md.count.L) = Moff{jj}(1:end-1);
    M(st+md.count.L:step:fin-1) = Moff{jj}(1:end-1);
    
    K(st:step:fin) = Kcen{jj}(1:end-1);
    K(st+1:step:fin-md.count.L) = Koff{jj}(1:end-1);
    K(st+md.count.L:step:fin-1) = Koff{jj}(1:end-1);
    
end

% connections

for ii = 1:length(md.par)
   
    jj = md.par(ii);
    
    segh = md.segh(jj);
    d1 = nc.treedata{jj}(1);
    d2 = nc.treedata{jj}(2);
    
    segt1 = md.segt(d1);
    segt2 = md.segt(d2);
    
    M(segh,segh) = M(segh,segh) + Mcen{d1}(end) + Mcen{d2}(end); 
    M(segh,segt1) = Moff{d1}(end);
    M(segt1,segh) = Moff{d1}(end);
    M(segh,segt2) = Moff{d2}(end);
    M(segt2,segh) = Moff{d2}(end);
    
    K(segh,segh) = K(segh,segh) + Kcen{d1}(end) + Kcen{d2}(end); 
    K(segh,segt1) = Koff{d1}(end);
    K(segt1,segh) = Koff{d1}(end);
    K(segh,segt2) = Koff{d2}(end);
    K(segt2,segh) = Koff{d2}(end);
     
end

% make the block diagonal matrix

%A = spalloc(md.count.L*3,md.count.L*3,3*(md.count.L*3-4));

kBp = p_const.Ca.kBp;
kEp = p_const.Ca.kEp;
kBm = p_const.Ca.kBm;
kEm = p_const.Ca.kEm;

Bt = p_const.Ca.Bt;
Et = p_const.Ca.Et;

Dc = p_const.Ca.Dc;
DB = p_const.Ca.DB;
DE = p_const.Ca.DE;

N = md.count.L;

Ca_A = [kBp*Bt*M+kEp*Et*M+Dc*K  -kBm*M          -kEm*M;
        -kBp*Bt*M                   kBm*M+DB*K      zeros(N,N);
        -kEp*Et*M                   zeros(N,N)      kEm*M+DE*K];
 
Ca_M = [M zeros(N,N) zeros(N,N); zeros(N,N) M zeros(N,N);
        zeros(N,N) zeros(N,N) M];

% make the hybridized RHS vector

