% neur_loadmorph(nc,compmaxsize)
% 
% Assigns parameters for the compartments
%
% inputs: nc : structure containing morphology data from trace file
%         
%         compmaxsize : maximum compartment size in (um)
%
% outputs: md : structure containing compartment parameters
%
%          md.count.N : number of dendritic branches in the cell
%          md.count.L : number of compartments total in the cell
%
%          (not currently used)
%          md.interp.radii : radii of each compartment
%          md.interp.size  : length of each compartment

function md = neur_loadmorph(nc,compmaxsize)

md.count.N = length(nc.celldata);

md.parents = zeros(md.count.N,1);

p = 1;
paryes = 0;

% traverses through all of the branches to md.interpolate radii

for j = 1:md.count.N         
    
    % finds physical length of the current branch
    
    md.seglength(j) = nc.lendata{j}(nc.segsize(j))*1e-4;   
    
    % md.interpolates radii for the current segment
    
    % number of compartments needed
    
    comps(j) = ceil(md.seglength(j)/compmaxsize);       
    
    % compartment length calculated
    
    md.hstep(j) = md.seglength(j)/comps(j);            
        
    md.interp.radii{j} = 1e-4*interp1(nc.lendata{j},...
                        (nc.celldata{j}(1:nc.segsize(j),4))',...
                         0:md.hstep(j):md.seglength(j))/2;
                         
    md.interp.size(j) = length(md.interp.radii{j});
    
    % find parents

    if size(nc.treedata{j},2) > 0
        md.par(p) = j;
        p = p + 1;
        paryes = 1;
        
        md.parents(nc.treedata{j}(1)) = j;
        md.parents(nc.treedata{j}(2)) = j;
    end    
    
    % find largest radius
    
    md.maxrad(j) = max(nc.celldata{j}(1:nc.segsize(j),4));
    
end

[md.interp.rsize, md.interp.gsize] = neur_morphresize(nc,md);

md.count.L = sum(md.interp.size)-size(md.interp.size,2)+1;

if paryes 
    md.count.P = size(md.par,2);
else
    md.count.P = 0;
end

% find largest radius

md.maxrad = max(md.maxrad);

function [rsize,gsize] = neur_morphresize(nc,md)

for ii = 1:md.count.N

    % make tempsize array (adjust for triple-click)
    
    rsize(ii) = md.interp.size(ii)-1;
    gsize(ii) = rsize(ii);
    fr = nc.roots(find(nc.roots == ii));
    rsize(fr) = rsize(fr) + 1;
end