%
% solve the passive axon problem via finite Fourier series
%
% usage:	paxon_ex(N,dt,T,pinc)
%
%             	N = number of patches
%               dt = timestep
%               T = stopping time
%               pinc = number of time steps between plots
%
% example:	paxon_ex(64,.05,6,4)
%

function V = paxon_ex(N,dt,T,pinc)

a = 0.01;		% cm
ell = 1;		% cm
C_M = 1;		% micro F / cm^2
g_L = 0.3;     		% mS / cm^2
R_2 = 0.034;		% k Ohm cm
G_2 = a/2/R_2;		% scaled axial conductance
dx = ell/N;		% patch length
x = (0:dx:ell)';	        % vector of patch midpoints

V(:,1) = zeros(size(x));		% initial conditions

t = 0;
tcnt = 0;
%plot3(x,t*ones(N,1),V(:,1))
%drawnow
%hold on

while (t <= T)

     t = t + dt

     V(:,tcnt+2) = zeros(size(x));

     for n=0:1000,

         an = (G_2*(n*pi/ell)^2+g_L)/C_M;

         V(:,tcnt+2) = V(:,tcnt+2) + ...
             cos(n*pi*x/ell)*(exp(-t)*((an-1)*t-1)+exp(-an*t))/(an-1)^2;
         if n==0,
            V(:,tcnt+2) = V(:,tcnt+2)/2;
         end

     end

     V(:,tcnt+2) = V(:,tcnt+2)/C_M/a/pi/ell;

      tcnt = tcnt + 1;

%       if mod(tcnt,pinc) == 0
%          plot3(x,t*ones(N,1),V(:,tcnt))
%          drawnow
%       end
end

% xlabel('x (cm)','fontsize',16)
% ylabel('t (ms)','fontsize',16)
% zlabel('V (mV)','fontsize',16)

%hold off
